@@ -1,6 +1,6 @@
 /*
  * Chromaprint -- Audio fingerprinting toolkit
- * Copyright (C) 2010  Lukas Lalinsky <lalinsky@gmail.com>
+ * Copyright (C) 2010-2012  Lukas Lalinsky <lalinsky@gmail.com>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -45,14 +45,15 @@ extern "C" {
 
 typedef void *ChromaprintContext;
 
-#define CHROMAPRINT_VERSION_MAJOR 0
-#define CHROMAPRINT_VERSION_MINOR 5
+#define CHROMAPRINT_VERSION_MAJOR 1
+#define CHROMAPRINT_VERSION_MINOR 1
 #define CHROMAPRINT_VERSION_PATCH 0
 
 enum ChromaprintAlgorithm {
 	CHROMAPRINT_ALGORITHM_TEST1 = 0,
 	CHROMAPRINT_ALGORITHM_TEST2,
-	CHROMAPRINT_ALGORITHM_TEST3
+	CHROMAPRINT_ALGORITHM_TEST3,
+	CHROMAPRINT_ALGORITHM_TEST4
 };
 
 #define CHROMAPRINT_ALGORITHM_DEFAULT CHROMAPRINT_ALGORITHM_TEST2
@@ -65,6 +66,10 @@ CHROMAPRINT_API const char *chromaprint_get_version(void);
 /**
  * Allocate and initialize the Chromaprint context.
  *
+ * Note that when Chromaprint is compiled with FFTW, this function is
+ * not reentrant and you need to call it only from one thread at a time.
+ * This is not a problem when using FFmpeg or vDSP.
+ *
  * Parameters:
  *  - version: Version of the fingerprint algorithm, use
  *             CHROMAPRINT_ALGORITHM_DEFAULT for the default
@@ -78,6 +83,10 @@ CHROMAPRINT_API ChromaprintContext *chromaprint_new(int algorithm);
 /**
  * Deallocate the Chromaprint context.
  *
+ * Note that when Chromaprint is compiled with FFTW, this function is
+ * not reentrant and you need to call it only from one thread at a time.
+ * This is not a problem when using FFmpeg or vDSP.
+ *
  * Parameters:
  *  - ctx: Chromaprint context pointer
  */
@@ -88,6 +97,25 @@ CHROMAPRINT_API void chromaprint_free(ChromaprintContext *ctx);
  */
 CHROMAPRINT_API int chromaprint_get_algorithm(ChromaprintContext *ctx);
 
+/**
+ * Set a configuration option for the selected fingerprint algorithm.
+ *
+ * NOTE: DO NOT USE THIS FUNCTION IF YOU ARE PLANNING TO USE
+ * THE GENERATED FINGERPRINTS WITH THE ACOUSTID SERVICE.
+ *
+ * Parameters:
+ *  - ctx: Chromaprint context pointer
+ *  - name: option name
+ *  - value: option value
+ *
+ * Possible options:
+ *  - silence_threshold: threshold for detecting silence, 0-32767
+ *
+ * Returns:
+ *  - 0 on error, 1 on success
+ */
+CHROMAPRINT_API int chromaprint_set_option(ChromaprintContext *ctx, const char *name, int value);
+
 /**
  * Restart the computation of a fingerprint with a new audio stream.
  *
