@@ -1,6 +1,6 @@
 /*
  * Chromaprint -- Audio fingerprinting toolkit
- * Copyright (C) 2010  Lukas Lalinsky <lalinsky@gmail.com>
+ * Copyright (C) 2010-2012  Lukas Lalinsky <lalinsky@gmail.com>
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -23,6 +23,7 @@
 
 #include "utils.h"
 #include "audio_consumer.h"
+#include "moving_average.h"
 
 namespace Chromaprint
 {
@@ -30,7 +31,7 @@ namespace Chromaprint
 	class SilenceRemover : public AudioConsumer
 	{
 	public:
-		SilenceRemover(AudioConsumer *consumer);
+		SilenceRemover(AudioConsumer *consumer, int threshold = 0);
 
 		AudioConsumer *consumer() const
 		{
@@ -46,10 +47,22 @@ namespace Chromaprint
 		void Consume(short *input, int length);
 		void Flush();
 
+		int threshold()
+		{
+			return m_threshold;
+		}
+
+		void set_threshold(int value)
+		{
+			m_threshold = value;
+		}
+
 	private:
 		CHROMAPRINT_DISABLE_COPY(SilenceRemover);
 
 		bool m_start;
+		int m_threshold;
+		MovingAverage<short> m_average;
 		AudioConsumer *m_consumer;
 	};
 
